library(ResistanceGA)
library(raster)
library(rgdal)
library(gdistance)

########################## SINGLE SURFACE OPTIMIZATION ########################################
work.direct<-getwd()

#create folder to store results
dir.create(paste0(work.direct,"/result_files"))
result.files<-paste0(work.direct,"/result_files/")

#import all .tif files in the specified directory
dir<-list.files(path=getwd(), pattern="\\.tif$")

for (i in 1:length(dir)){
  if (i==1){
    r<-raster(dir[i])
    r.stack<-stack(r)
	
  } else{
    r1<-raster(dir[i])
    r.stack<-stack(r.stack,r1)
  }
}

#check if they aligned and have the same CRS
#should print TRUE if aligned, otherwise it will print an error
compareRaster(r.stack[[1]],r.stack[[2:length(r.stack@layers)]])

###if compareRaster() yields an error, uncomment and run the code below to align raster files with the same CRS; assumes that rasters are in the same directory

# rasters_toAlign<-list.files(path=getwd(), pattern="\\.tif$") #list files with specific extension
# 
# r.stack<-NULL
# 
# for (i in 1:length(rasters_toAlign)){
#   if (i==1){
#     r1<-raster(rasters_toAlign[i]) #assumes the first raster will be used as reference
#     r.stack<-stack(r1)
#     
#   } else{
#     r2<-raster(rasters_toAlign[i])
#     r2.new <- resample(r2, r1, "bilinear")
#     r2.new2 <- mask(r2.new, r1)
#     r.stack<-stack(r.stack,r2.new2)
#   }
# }

###end of code to align rasters

#import file containing sampling points
points.raw<-read.table("points_larviparous.txt", header=T, sep="")
points1 <- SpatialPoints(points.raw[,c(2,3)], 
                        proj4string=CRS("+proj=utm +zone=29 +datum=WGS84 +units=m +no_defs +ellps=WGS84 +towgs84=0,0,0" )) 

#import pairwise genetic distance matrix
gen.dist<-as.vector(as.dist(read.table("larviparous_gdist.txt", header=F, sep="")))

####ResistanceGA
gdist.input <- gdist.prep(length(points1), #number of populations
                          samples = points1, #matrix of SpatialPoints object
                          response = gen.dist, #genetic distances as vector
                          method = 'commuteDistance') ## Optimize using commute distance

#create a list with all transformations to be applied to rasters in the r.stack object
#this vector follows the alphabetic order of the rasters included in the r.stack
trans.vector<-c(NA,rep("A",3), NA, rep("A",2), NA, rep("A",2), 
                NA, rep("A",3), NA, rep("A",2))
trans.list<-list(NULL)
trans.list[1:length(trans.vector)]<-trans.vector

#GA.prep function
GA.input<-GA.prep(ASCII.dir = r.stack,
                  Results.dir = result.files,
                  max.cont = 2000,
                  max.cat = 2000,
                  select.trans=trans.list,
                  method="AIC", 
                  maxiter = 500,
                  run=25,
                  #parallel=8, #controls the number of cores to be used in processing
                  pmutation = 0.25,
                  pcrossover = 0.9)

#single surface optimization				  
SS_RESULTS.gdist <- SS_optim(gdist.inputs = gdist.input,
                             GA.inputs = GA.input)

########################## MULTIPLE SURFACE OPTIMIZATION ########################################
#the code is similar to the one displayed in single surface optimization
#the only difference is all.comb.results() is run instead of SS_optim() and some parameters changed in GA.prep
#this example assumes one categorical and one continuous rasters were included in all_comb()
GA.input<-GA.prep(ASCII.dir = r.stack,
                  Results.dir = "all_comb",
                  max.cont = 2000,
                  max.cat = 2000,
                  select.trans=list(NA,"A"), #transformations to be applied to each raster 
                  method="AIC", 
                  maxiter = 200,
                  run=25,
                  #parallel=8,
                  pmutation = 0.25,
                  pcrossover = 0.9)
				  
all.comb.results<-all_comb(gdist.inputs = gdist.input,
                           GA.inputs = GA.input,
                           results.dir = "all_comb",
                           max.combination = 2,
                           replicate = 1,
                           dist_mod=F,
                           null_mod = F)